#pragma once
#include "token.hpp"

namespace jlog {
    /// Returns a string timestamp in the format of 'YYYY-MM-DD hh:mm:ss.ms'
    std::string get_timestamp();

    /// Returns a pseudo-"stacktrace" formatted sequence of tokens.
    /// @param func The function name/signature to trace back to. Should be provided by a __FUNCTION__ macro variant.
    /// @param file The file name to trace back to. Should be provided by a __FILE__ macro variant.
    /// @param line The source-code line to trace back to. Should be provided by a __LINE__ macro variant.
        std::vector<token> trace_format(
                const std::string &func,
                const std::string &file,
                int line);

    /// Returns a formatted sequence of tokens given a severity and message.
    /// @param severity_name The severity tag to prefix to the message. Could theoretically also be a context.
    /// @param message The actual message to include
    /// @param severity_cc The colorcode to assign to the severity.
        std::vector<token> log_format(
                const std::string &severity_name,
                const std::string &message,
                const Color4 &severity_cc = Colors::White);

    /// Returns a more detailed formatted sequence of tokens.
    /// @param severity_name The severity tag to prefix to the message. Could theoretically also be a context.
    /// @param message The actual message to include
    /// @param func The function name/signature to trace back to. Should be provided by a __FUNCTION__ macro variant.
    /// @param file The file name to trace back to. Should be provided by a __FILE__ macro variant.
    /// @param line The source-code line to trace back to. Should be provided by a __LINE__ macro variant.
    /// @param severity_cc The colorcode to assign to the severity.
        std::vector<token> log_format(
                const std::string &severity_name,
                const std::string &message,
                const std::string &func,
                const std::string &file,
                int line,
                const Color4 &severity_cc = Colors::White);

    /// Returns a token sequence pre-formatted for the info LogTrace level.
    /// @param message The message to send out.
        std::vector<token> info_format(const std::string &message);

    /// Returns a token sequence pre-formatted for the info LogTrace level.
    /// @param message The message to send out.
    /// @param func The function name/signature to trace back to. Should be provided by a __FUNCTION__ macro variant.
    /// @param file The file name to trace back to. Should be provided by a __FILE__ macro variant.
    /// @param line The source-code line to trace back to. Should be provided by a __LINE__ macro variant.
        std::vector<token> info_detailed_format(
                const std::string &message,
                const std::string &func,
                const std::string &file,
                int line);

    /// Returns a token sequence pre-formatted for the warning LogTrace level.
    /// @param message The message to send out.
        std::vector<token> warning_format(const std::string &message);

    /// Returns a token sequence pre-formatted for the warning LogTrace level.
    /// @param message The message to send out.
    /// @param func The function name/signature to trace back to. Should be provided by a __FUNCTION__ macro variant.
    /// @param file The file name to trace back to. Should be provided by a __FILE__ macro variant.
    /// @param line The source-code line to trace back to. Should be provided by a __LINE__ macro variant.
        std::vector<token> warning_detailed_format(
                const std::string &message,
                const std::string &func,
                const std::string &file,
                int line);

    /// Returns a token sequence pre-formatted for the error LogTrace level.
    /// @param message The message to send out.
        std::vector<token> error_format(const std::string &message);

    /// Returns a token sequence pre-formatted for the error LogTrace level.
    /// @param message The message to send out.
    /// @param func The function name/signature to trace back to. Should be provided by a __FUNCTION__ macro variant.
    /// @param file The file name to trace back to. Should be provided by a __FILE__ macro variant.
    /// @param line The source-code line to trace back to. Should be provided by a __LINE__ macro variant.
        std::vector<token> error_detailed_format(
                const std::string &message,
                const std::string &func,
                const std::string &file,
                int line);

    /// Returns a token sequence pre-formatted for the fatal LogTrace level.
    /// @param message The message to send out.
        std::vector<token> fatal_format(const std::string &message);

    /// Returns a token sequence pre-formatted for the fatal LogTrace level.
    /// @param message The message to send out.
    /// @param func The function name/signature to trace back to. Should be provided by a __FUNCTION__ macro variant.
    /// @param file The file name to trace back to. Should be provided by a __FILE__ macro variant.
    /// @param line The source-code line to trace back to. Should be provided by a __LINE__ macro variant.
        std::vector<token> fatal_detailed_format(
                const std::string &message,
                const std::string &func,
                const std::string &file,
                int line);

    /// Returns a token sequence pre-formatted for the verbose LogTrace level.
    /// @param message The message to send out.
        std::vector<token> verbose_format(const std::string &message);

    /// Returns a token sequence pre-formatted for the verbose LogTrace level.
    /// @param message The message to send out.
    /// @param func The function name/signature to trace back to. Should be provided by a __FUNCTION__ macro variant.
    /// @param file The file name to trace back to. Should be provided by a __FILE__ macro variant.
    /// @param line The source-code line to trace back to. Should be provided by a __LINE__ macro variant.
        std::vector<token> verbose_detailed_format(
                const std::string &message,
                const std::string &func,
                const std::string &file,
                int line);

    /// Returns a token sequence pre-formatted for the debug LogTrace level.
    /// @param message The message to send out.
        std::vector<token> debug_format(const std::string &message);

    /// Returns a token sequence pre-formatted for the debug LogTrace level.
    /// @param message The message to send out.
    /// @param func The function name/signature to trace back to. Should be provided by a __FUNCTION__ macro variant.
    /// @param file The file name to trace back to. Should be provided by a __FILE__ macro variant.
    /// @param line The source-code line to trace back to. Should be provided by a __LINE__ macro variant.
        std::vector<token> debug_detailed_format(
                const std::string &message,
                const std::string &func,
                const std::string &file,
                int line);
}